<?php

namespace Drupal\designs_view\Plugin\views\row;

use Drupal\Core\Form\FormStateInterface;
use Drupal\designs\DesignContentManagerInterface;
use Drupal\designs\DesignManagerInterface;
use Drupal\designs\DesignSettingManagerInterface;
use Drupal\designs\Form\PluginForm;
use Drupal\views\Plugin\views\row\RowPluginBase;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * The 'design' row plugin.
 *
 * This displays fields using a design.
 *
 * @ingroup views_row_plugins
 *
 * @ViewsRow(
 *   id = "design",
 *   title = @Translation("Design"),
 *   help = @Translation("Displays the fields using a design."),
 * )
 */
class DesignsRow extends RowPluginBase {

  /**
   * {@inheritdoc}
   */
  protected $usesFields = TRUE;

  /**
   * The design manager.
   *
   * @var \Drupal\designs\DesignManagerInterface
   */
  protected DesignManagerInterface $designManager;

  /**
   * The design setting manager.
   *
   * @var \Drupal\designs\DesignSettingManagerInterface
   */
  protected DesignSettingManagerInterface $settingManager;

  /**
   * The design content manager.
   *
   * @var \Drupal\designs\DesignContentManagerInterface
   */
  protected DesignContentManagerInterface $contentManager;

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    $instance = parent::create($container, $configuration, $plugin_id, $plugin_definition);
    $instance->designManager = $container->get('plugin.manager.design');
    $instance->settingManager = $container->get('plugin.manager.design_setting');
    $instance->contentManager = $container->get('plugin.manager.design_content');
    return $instance;
  }

  /**
   * {@inheritdoc}
   */
  protected function defineOptions() {
    $options = parent::defineOptions();
    $options['design'] = [
      'default' => [
        'design' => '',
        'settings' => [],
        'content' => [],
        'regions' => [],
      ],
    ];
    return $options;
  }

  /**
   * Provide a form for setting options.
   */
  public function buildOptionsForm(&$form, FormStateInterface $form_state) {
    parent::buildOptionsForm($form, $form_state);

    /** @var \Drupal\views\ViewExecutable $view */
    $view = $form_state->get('view')->getExecutable();

    $design_form = new PluginForm(
      $this->designManager,
      $this->settingManager,
      $this->contentManager,
      $this->options['design']['design'],
      $this->options['design'],
      'views_row',
      $view->display_handler->getFieldLabels(),
    );

    $form['design'] = [
      '#parents' => ['row_options', 'design'],
    ];
    $form['design'] = $design_form->buildForm($form['design'], $form_state);
    $form['design']['#type'] = 'container';
  }

  /**
   * {@inheritdoc}
   */
  public function validateOptionsForm(&$form, FormStateInterface $form_state) {
    parent::validateOptionsForm($form, $form_state);
    $form['design']['#form_handler']->validateForm($form['design'], $form_state);
  }

  /**
   * {@inheritdoc}
   */
  public function submitOptionsForm(&$form, FormStateInterface $form_state) {
    parent::submitOptionsForm($form, $form_state);
    $form['design']['#form_handler']->submitForm($form['design'], $form_state);
  }

  /**
   * {@inheritdoc}
   */
  public function render($row) {
    $result = parent::render($row);

    $options = $this->options['design'];
    if (empty($options['design'])) {
      return $result;
    }

    $design = $this->designManager->createSourcedInstance(
      $options['design'],
      $options,
      'views_row',
      $this->view->display_handler->getFieldLabels(),
    );
    if (!$design) {
      return $result;
    }

    // Since most of the rendering is in template_preprocess_views_view_field()
    // We emulate the general behaviour of the fields here as render elements.
    $element = [
      '#view' => $this->view,
      '#row' => $row,
    ];
    foreach ($this->view->field as $id => $field) {
      $field_output = $this->view->style_plugin->getField($row->index, $id);
      $empty = $field->isValueEmpty($field_output, $field->options['empty_zero']);
      if (empty($field->options['exclude']) && (!$empty || (empty($field->options['hide_empty']) && empty($variables['options']['hide_empty'])))) {
        $element[$id] = [
          '#markup' => $field_output,
        ];
      }
    }

    return $design->build($element);
  }

  /**
   * {@inheritdoc}
   */
  public function themeFunctions() {
    return $this->view->buildThemeFunctions('views_view_fields');
  }

}
