<?php

namespace Drupal\designs_view\Plugin\views\style;

use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Plugin\ContainerFactoryPluginInterface;
use Drupal\designs\DesignContentManagerInterface;
use Drupal\designs\DesignManagerInterface;
use Drupal\designs\DesignSettingManagerInterface;
use Drupal\designs\Form\PluginForm;
use Drupal\views\Plugin\views\style\StylePluginBase;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * A Views style that renders a design.
 *
 * @ingroup views_style_plugins
 *
 * @ViewsStyle(
 *   id = "designs",
 *   title = @Translation("Design"),
 *   help = @Translation("Uses a design to arrange views elements."),
 * )
 */
class DesignsStyle extends StylePluginBase implements ContainerFactoryPluginInterface {

  /**
   * @var bool
   */
  protected $usesRowPlugin = TRUE;

  /**
   * @var bool
   */
  protected $usesGrouping = FALSE;

  /**
   * @var bool
   */
  protected $usesRowClass = FALSE;

  /**
   * @var bool
   */
  protected $usesFields = TRUE;

  /**
   * @var bool
   */
  protected $usesOptions = TRUE;

  /**
   * The design manager.
   *
   * @var \Drupal\designs\DesignManagerInterface
   */
  protected DesignManagerInterface $designManager;

  /**
   * The design setting manager.
   *
   * @var \Drupal\designs\DesignSettingManagerInterface
   */
  protected DesignSettingManagerInterface $settingManager;

  /**
   * The design content manager.
   *
   * @var \Drupal\designs\DesignContentManagerInterface
   */
  protected DesignContentManagerInterface $contentManager;

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    $instance = parent::create($container, $configuration, $plugin_id, $plugin_definition);
    $instance->designManager = $container->get('plugin.manager.design');
    $instance->settingManager = $container->get('plugin.manager.design_setting');
    $instance->contentManager = $container->get('plugin.manager.design_content');
    return $instance;
  }

  /**
   * {@inheritdoc}
   */
  protected function defineOptions() {
    $options = parent::defineOptions();
    $options['design'] = [
      'default' => [
        'design' => '',
        'settings' => [],
        'content' => [],
        'regions' => [],
      ],
    ];
    return $options;
  }

  /**
   * {@inheritdoc}
   */
  public function buildOptionsForm(&$form, FormStateInterface $form_state) {
    parent::buildOptionsForm($form, $form_state);
    $design_form = new PluginForm(
      $this->designManager,
      $this->settingManager,
      $this->contentManager,
      $this->options['design']['design'],
      $this->options['design'],
      'views_style',
      [],
    );

    $form['design'] = [
      '#parents' => ['style_options', 'design'],
      '#submit' => [[$form_state->getFormObject(), 'submitForm']],
    ];
    $form['design'] = $design_form->buildForm($form['design'], $form_state);
    $form['design']['#type'] = 'container';
  }

  /**
   * {@inheritdoc}
   */
  public function validateOptionsForm(&$form, FormStateInterface $form_state) {
    parent::validateOptionsForm($form, $form_state);
    $form['design']['#form_handler']->validateForm($form['design'], $form_state);
  }

  /**
   * {@inheritdoc}
   */
  public function submitOptionsForm(&$form, FormStateInterface $form_state) {
    parent::submitOptionsForm($form, $form_state);
    $form['design']['#form_handler']->submitForm($form['design'], $form_state);
  }

  /**
   * {@inheritdoc}
   */
  public function render() {
    $rows = [];
    $row_plugin = $this->usesRowPlugin();
    foreach ($this->view->result as $index => $row) {
      $this->view->row_index = $index;
      $rows[$index] = $row_plugin ? $this->view->rowPlugin->render($row) : $row;
    }
    unset($this->view->row_index);

    return [
      '#design' => $this->options['design'],
    ] + $rows;
  }

  /**
   * {@inheritdoc}
   */
  public function themeFunctions() {
    return $this->view->buildThemeFunctions('views_view_design');
  }

}
